#include	"myHeader.h"

//	Most of the codes shown in this file "p_value.c" is taken from Okumura H, "Dictionary of Latest Algorithms Written in C Language (1991)" Published by Gijutsu Hyoron Sha, Tokyo, Japan.
//	ISBN code of the book is ISBN4-87408-414-1 C3055

#define PI      3.14159265358979324  // $\pi$ 
#define LOG_2PI 1.83787706640934548  // $\log 2\pi$ 
#define N       8
#define LOG_PI 1.14472988584940017  // $\log_e \pi$ 

#define B0  1                 // Bernoulli number
#define B1  (-1.0 / 2.0)
#define B2  ( 1.0 / 6.0)
#define B4  (-1.0 / 30.0)
#define B6  ( 1.0 / 42.0)
#define B8  (-1.0 / 30.0)
#define B10 ( 5.0 / 66.0)
#define B12 (-691.0 / 2730.0)
#define B14 ( 7.0 / 6.0)
#define B16 (-3617.0 / 510.0)

double p_gamma(double a, double x, double loggamma_a);
double q_gamma(double a, double x, double loggamma_a);
double loggamma(double x); //logarithm of Gamma function
double p_chisq(double chisq, int df);  // low side probability of chi-square distribution
double q_chisq(double chisq, int df);  // 
double erf(double x);  // Gaussian error function
double erfc(double x); 
double p_normal(double x);  // low side probability of normal distribution
double q_normal(double x);  // high  side probability of normal distribution

double	p_value ( int sampleNo1, int sampleNo2, int positiveNo1, int positiveNo2 )
{
	double	u, n1, n2, x1, x2, p1starbar, p2starbar, pstarbar, q;
	
	n1 = (double)sampleNo1;
	n2 = (double)sampleNo2;
	x1 = (double)positiveNo1;
	x2 = (double)positiveNo2;
	
	p1starbar = ( x1 + 0.5 ) / ( n1 + 1.0 );
	p2starbar = ( x2 + 0.5 ) / ( n2 + 1.0 );
	pstarbar = ( x1 + x2 + 0.5 ) / ( n1 + n2 + 1.0 );
	
	u = ( p1starbar - p2starbar ) / sqrt ( pstarbar * ( 1.0 - pstarbar ) * ( 1.0 / n1 + 1.0 / n2 ) );
	q = q_normal( u ); 
	return ( q );
}

double loggamma( double x )  //logarithm of Gamma function
{
	double v, w;

	v = 1;
	while ( x < N ) {  v *= x;  x++;  }
	w = 1 / ( x * x );
	return (((((((( B16 / ( 16 * 15))  * w + ( B14 / (14 * 13))) * w
	            + ( B12 / ( 12 * 11))) * w + ( B10 / (10 *  9))) * w
	            + ( B8  / (  8 *  7))) * w + ( B6  / ( 6 *  5))) * w
	            + ( B4  / (  4 *  3))) * w + ( B2  / ( 2 *  1))) / x
	            + 0.5 * LOG_2PI - log( v ) - x + ( x - 0.5 ) * log( x );
}

double p_gamma(double a, double x, double loggamma_a)
{
	int k;
	double result, term, previous;

	if ( x >= 1 + a ) return 1 - q_gamma( a, x, loggamma_a );
	if ( x == 0 )     return 0;
	result = term = exp(a * log( x ) - x - loggamma_a ) / a;
	for ( k = 1; k < 1000; k++ ) {
		term *= x / ( a + k );
		previous = result;  result += term;
		if (result == previous) return result;
	}
	printf("p_gamma(): Do not converge\n");
	return result;
}

double q_gamma(double a, double x, double loggamma_a) 
{
	int k;
	double result, w, temp, previous;
	double la = 1, lb = 1 + x - a;  // Laguerre's polynomial equation

	if ( x < 1 + a ) return 1 - p_gamma( a, x, loggamma_a );
	w = exp( a * log( x ) - x - loggamma_a );
	result = w / lb;
	for ( k = 2; k < 1000; k++ ) {
		temp = ( ( k - 1 - a ) * ( lb - la ) + ( k + x ) * lb) / k;
		la = lb;  lb = temp;
		w *= ( k - 1 - a ) / k;
		temp = w / ( la * lb );
		previous = result;  result += temp;
		if (result == previous) return result;
	}
	printf("q_gamma(): Do not converge\n");
	return result;
}

double p_chisq(double chisq, int df) 
{
	return p_gamma( 0.5 * df, 0.5 * chisq, loggamma( 0.5 * df ) );
}

double q_chisq(double chisq, int df)  
{
	return q_gamma( 0.5 * df, 0.5 * chisq, loggamma( 0.5 * df ) );
}

double erf(double x)  // Gaussian error function
{
	if ( x >= 0 ) return   p_gamma( 0.5, x * x, LOG_PI / 2 );
	else        return - p_gamma( 0.5, x * x, LOG_PI / 2 );
}

double erfc(double x) 
{
	if ( x >= 0 ) return  q_gamma( 0.5, x * x, LOG_PI / 2);
	else  return  1 + p_gamma( 0.5, x * x, LOG_PI / 2);
}

double p_normal(double x) 
{
	if ( x >= 0 ) return
		0.5 * (1 + p_gamma( 0.5, 0.5 * x * x, LOG_PI / 2));
	else return
		0.5 * q_gamma( 0.5, 0.5 * x * x, LOG_PI / 2);
}

double q_normal(double x) 
{
	if ( x >= 0 ) return
		0.5 * q_gamma( 0.5, 0.5 * x * x, LOG_PI / 2 );
	else return
		0.5 * ( 1 + p_gamma( 0.5, 0.5 * x * x, LOG_PI / 2 ) );
}
